@@ -1,6 +1,7 @@
 ;; geiser-guile.el -- guile's implementation of the geiser protocols
 
-;; Copyright (C) 2009, 2010, 2011, 2012, 2013 Jose Antonio Ortega Ruiz
+;; Copyright (C) 2009-2018 Jose Antonio Ortega Ruiz
+;; Copyright (C) 2017 Jan Nieuwenhuizen <janneke@gnu.org>
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the Modified BSD License. You should
@@ -41,8 +42,10 @@
   :group 'geiser-guile)
 
 (geiser-custom--defcustom geiser-guile-load-path nil
-  "A list of paths to be added to Guile's load path when it's
-started."
+  "A list of paths to be added to Guile's load path when it's started.
+The paths are added to both %load-path and %load-compiled path,
+and only if they are not already present.  This variable is a
+good candidate for an entry in your project's .dir-locals.el."
   :type '(repeat file)
   :group 'geiser-guile)
 
@@ -171,9 +174,7 @@ This function uses `geiser-guile-init-file' if it exists."
 (defun geiser-guile--get-module (&optional module)
   (cond ((null module)
          (save-excursion
-           (ignore-errors
-             (while (not (zerop (geiser-syntax--nesting-level)))
-               (backward-up-list)))
+           (geiser-syntax--pop-to-top)
            (if (or (re-search-backward geiser-guile--module-re nil t)
                    (looking-at geiser-guile--library-re)
                    (re-search-forward geiser-guile--module-re nil t))
@@ -206,7 +207,7 @@ This function uses `geiser-guile-init-file' if it exists."
   (if module
       (max (save-excursion (beginning-of-line) (point))
            (save-excursion (skip-syntax-backward "^(>") (1- (point))))
-    (save-excursion (skip-syntax-backward "^-()>") (point))))
+    (save-excursion (skip-syntax-backward "^'-()>") (point))))
 
 
 ;;; Error display
@@ -226,11 +227,10 @@ This function uses `geiser-guile-init-file' if it exists."
       (ignore-errors (next-error)))))
 
 (defun geiser-guile--display-error (module key msg)
-  (newline)
   (when (stringp msg)
     (save-excursion (insert msg))
     (geiser-edit--buttonize-files))
-  (and (not key) msg (not (zerop (length msg)))))
+  (and (not key) (not (zerop (length msg))) msg))
 
 
 ;;; Trying to ascertain whether a buffer is Guile Scheme:
@@ -247,22 +247,67 @@ This function uses `geiser-guile-init-file' if it exists."
 
 ;;; Keywords and syntax
 
+(defconst geiser-guile--builtin-keywords
+  '("call-with-input-file"
+    "call-with-input-string"
+    "call-with-output-file"
+    "call-with-output-string"
+    "call-with-prompt"
+    "call-with-trace"
+    "define-accessor"
+    "define-class"
+    "define-enumeration"
+    "define-inlinable"
+    "define-syntax-parameter"
+    "eval-when"
+    "lambda*"
+    "syntax-parameterize"
+    "use-modules"
+    "with-error-to-file"
+    "with-error-to-port"
+    "with-error-to-string"
+    "with-fluid*"
+    "with-fluids"
+    "with-fluids*"
+    "with-input-from-port"
+    "with-input-from-string"
+    "with-output-to-port"
+    "with-output-to-string"))
+
 (defun geiser-guile--keywords ()
-  (when geiser-guile-extra-keywords
-    `((,(format "[[(]%s\\>" (regexp-opt geiser-guile-extra-keywords 1))
-       . 1))))
+  (append
+   (geiser-syntax--simple-keywords geiser-guile-extra-keywords)
+   (geiser-syntax--simple-keywords geiser-guile--builtin-keywords)
+   `((,(rx "(" (group "define-once") eow (* space) (? (group (+ word))))
+       (1 font-lock-keyword-face)
+       (2 font-lock-variable-name-face nil t))
+     ("(\\(define-module\\) +(\\([^)]+\\))"
+      (1 font-lock-keyword-face)
+      (2 font-lock-type-face nil t)))))
 
 (geiser-syntax--scheme-indent
  (c-declare 0)
  (c-lambda 2)
+ (call-with-input-string 1)
+ (call-with-output-string 0)
+ (call-with-prompt 1)
+ (call-with-trace 0)
+ (eval-when 1)
  (lambda* 1)
  (pmatch defun)
  (sigaction 1)
+ (syntax-parameterize 1)
+ (with-error-to-file 1)
+ (with-error-to-port 1)
+ (with-error-to-string 0)
  (with-fluid* 1)
  (with-fluids 1)
  (with-fluids* 1)
- (with-method 1))
-
+ (with-input-from-string 1)
+ (with-method 1)
+ (with-mutex 1)
+ (with-output-to-string 0)
+ (with-throw-handler 1))
 
 
 ;;; Compilation shell regexps
@@ -289,10 +334,10 @@ This function uses `geiser-guile-init-file' if it exists."
 
 ;;; REPL startup
 
-(defconst geiser-guile-minimum-version "2.0")
+(defconst geiser-guile-minimum-version "2.2")
 
 (defun geiser-guile--version (binary)
-  (shell-command-to-string (format "%s  -c '(display (version))'" binary)))
+  (car (process-lines binary "-c" "(display (version))")))
 
 (defun geiser-guile-update-warning-level ()
   "Update the warning level used by the REPL.
@@ -357,12 +402,11 @@ it spawn a server thread."
 (defun guile--manual-look-up (id mod)
   (let ((info-lookup-other-window-flag
          geiser-guile-manual-lookup-other-window-p))
-    (info-lookup-symbol id 'geiser-guile-mode))
+    (info-lookup-symbol (symbol-name id) 'scheme-mode))
   (when geiser-guile-manual-lookup-other-window-p
     (switch-to-buffer-other-window "*info*"))
   (search-forward (format "%s" id) nil t))
 
-
 
 ;;; Implementation definition:
 
